/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 * 
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.fabric.hc.util;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.List;
import java.util.Properties;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.webify.shared.edi.model.EDIInterchange;
import com.webify.shared.edi.model.EDIParseException;
import com.webify.shared.edi.model.LoopFunctionalGroup;
import com.webify.shared.edi.model.SegmentGS;
import com.webify.shared.edi.model.hipaa270.Hipaa270Document;
import com.webify.shared.edi.model.hipaa270.Loop2000A;
import com.webify.shared.edi.model.hipaa270.Loop2000B;
import com.webify.shared.edi.model.hipaa270.Loop2000C;
import com.webify.shared.edi.model.hipaa271.Hipaa271Document;
import com.webify.shared.edi.model.hipaa271.Loop2100C;
import com.webify.shared.edi.model.hipaa271.Loop2110C;
import commonj.sdo.DataObject;

/**
 * @author saldym
 * 
 */
public final class BPELUtility {

	private BPELUtility() {
	}

	private static final Log LOG = LogFactory.getLog(BPELUtility.class);

	/**
	 * Checks the TA1 acknowledgement generated after validating the interchange
	 * envelope of the incoming document.
	 * 
	 */
	public static boolean isEnvelopeValid(DataObject basicValidationObj) {
		DataObject ackDo = basicValidationObj.getDataObject("acknowledgement");
		LOG.info("STEP I " + new String(ackDo.getBytes("data")));

		String data = new String(ackDo.getBytes("data"));
		String status = data.substring(data.indexOf("~IEA") - 1, data
				.indexOf("~IEA"));
		LOG.info("Status " + status);
		if (status.equals("A")) {
			return true;
		}
		return false;
	}

	/**
	 * Checks the 997 acknowledgement generated after validating the incoming
	 * document.
	 * 
	 */
	public static boolean isDocumentValid(DataObject validationObj) {
		DataObject ackDo = validationObj.getDataObject("acknowledgement");
		String data = new String(ackDo.getBytes("data"));
		String status = data.substring(data.indexOf("AK5*") + 4, data
				.indexOf("AK5*") + 5);
		LOG.info("Status " + status);
		if (status.equals("A")) {
			return true;
		}
		return false;
	}

	/**
	 * Retrive the following member info from the current HIPAA270 document. 1)
	 * Last Name 2) First Name 3) Member Id 4) Date of Birth
	 */
	public static String[] getMemberInfo(DataObject unbundled270Obj,
			Integer counter) {
		String[] memberInfo = new String[4];
		int index = counter.intValue();
		if (unbundled270Obj != null) {
			DataObject edi270Obj = (DataObject) unbundled270Obj.getList(
					"ediFile").get(index);
			try {
				EDIInterchange ediInter = EDIInterchange.parse(edi270Obj
						.getBytes(0));
				Hipaa270Document hipaa270Doc = (Hipaa270Document) ((LoopFunctionalGroup) ediInter
						.getFunctionalGroups().get(0)).getDocument(0);
				Loop2000C[] loop2000C = getSubscriberLoop(hipaa270Doc);
				memberInfo[0] = loop2000C[0].getLoop2100C().getNm1().getNm103(); // lastName
				memberInfo[1] = loop2000C[0].getLoop2100C().getNm1().getNm104(); // firstName
				memberInfo[2] = loop2000C[0].getLoop2100C().getNm1().getNm109(); // memberId
				if (loop2000C[0].getLoop2100C().getDmg() != null) {
					memberInfo[3] = loop2000C[0].getLoop2100C().getDmg()
							.getDmg02(); // dateOfBirth
				} else {
					// add for dependent loop
					memberInfo[3] = "";
				}
			} catch (EDIParseException e) {
				LOG.error("This will happen if it a dependent BE request.", e);
			}
		}
		return memberInfo;
	}

	
	public static String getDate(String date) {
		String[] dateArray = StringUtils.split(date, '-');
		return dateArray[1]+"/"+dateArray[2]+"/"+dateArray[0];
	}
	/**
	 * Determines the LOB based on the memberId.
	 * 
	 */
	public static String getLOBAssertion(String memberId) {
		String lob = "Other";
		if (StringUtils.isNotBlank(memberId)) {
			if ("AMX".equals(memberId.substring(0, 3))) {
				lob = "EPO";
			} else if ("BMX".equals(memberId.substring(0, 3))) {
				lob = "HMO";
			}
		}
		return lob;
	}

	/**
	 * Determines the HIPAA DocumentType based on the content.
	 * 
	 */
	public static String getDocumenTypeAssertion(DataObject dataObject) {
		String documentType = null;
		if (dataObject != null) {
			DataObject ediObj = null;
			ediObj = (DataObject) dataObject.getDataObject("ediFile");
			documentType = getDocumentType(ediObj);

		} else {
			LOG.info("Inside else getDocumenTypeAssertion");
		}
		return documentType;
	}

	private static String getDocumentType(DataObject ediObj) {
		String documentType = null;
		try {
			EDIInterchange ediInter = EDIInterchange.parse(ediObj.getBytes(0));
			final String idCode = "004010X092A1";
			LoopFunctionalGroup funcGroup = (LoopFunctionalGroup) ediInter
					.getFunctionalGroups().get(0);
			SegmentGS gs = funcGroup.getGs();
			String gs01 = gs.getGs01();
			String gs08 = gs.getGs08();
			if ("HS".equals(gs01) && idCode.equals(gs08)) {
				documentType = "HIPAA270A1";
			} else if ("HB".equals(gs01) && idCode.equals(gs08)) {
				documentType = "HIPAA271A1";
			}
		} catch (EDIParseException e) {
			LOG.info("This should never happen." + e);
		}
		return documentType;
	}

	/**
	 * Increment the counter inside the while loop.
	 * 
	 */
	public static Integer incrementCount(Integer counter) {
		int cnt = counter.intValue() + 1;
		return new Integer(cnt);
	}

	/**
	 * Fetches the current Hipaa270 document from the while loop based on the
	 * counter.
	 * 
	 */
	public static byte[] getCurrentHipaa270Doc(DataObject unbundled270Obj,
			Integer counter) {
		byte[] ediFile = null;
		int index = counter.intValue();
		if (unbundled270Obj != null) {
			DataObject edi270Obj = (DataObject) unbundled270Obj.getList(
					"ediFile").get(index);
			try {
				EDIInterchange ediInter = EDIInterchange.parse(edi270Obj
						.getBytes(0));
				ediFile = ediInter.emit();

			} catch (EDIParseException e) {
				LOG.error("This should never happen.", e);
			}
		}
		return ediFile;
	}

	public static String[] getCoverageTypeDetails(byte[] data) {

		String[] coverageDetails = new String[5];
		try {
			EDIInterchange edi = EDIInterchange.parse(data);
			LoopFunctionalGroup functionalGrp = (LoopFunctionalGroup) edi
					.getFunctionalGroups().get(0);
			Hipaa271Document doc = (Hipaa271Document) ((LoopFunctionalGroup) edi
					.getFunctionalGroups().get(0)).getDocument(0);
			com.webify.shared.edi.model.hipaa271.Loop2000A loop2000A = (com.webify.shared.edi.model.hipaa271.Loop2000A) doc
					.getLoop2000A().get(0);
			if (loop2000A != null) {
				com.webify.shared.edi.model.hipaa271.Loop2000B loop2000B = (com.webify.shared.edi.model.hipaa271.Loop2000B) loop2000A
						.getLoop2000B().get(0);
				if (loop2000B != null) {
					com.webify.shared.edi.model.hipaa271.Loop2000C loop2000C = (com.webify.shared.edi.model.hipaa271.Loop2000C) loop2000B
							.getLoop2000C().get(0);
					if (loop2000C != null) {
						Loop2100C loop2100C = loop2000C.getLoop2100C();
						if (loop2100C != null) {
							List loop2100CList = loop2100C.getLoop2110C();
							if(loop2100CList == null) {
								 return null;
							} else {
							Loop2110C loop2110C = (Loop2110C) loop2100CList.get(0);
							if (loop2110C != null) {
								String eb1 = loop2110C.getEb().getEb01();
								String eb2 = loop2110C.getEb().getEb02();
								String eb3 = loop2110C.getEb().getEb03();
								String eb4 = loop2110C.getEb().getEb04();
								BigDecimal amount = loop2110C.getEb().getEb08();
/*								System.out.println("eb1 ="+eb1);
								System.out.println("eb2 ="+eb2);
								System.out.println("eb3 ="+eb3);
								System.out.println("eb4 ="+eb4);
								System.out.println("eb5 ="+amount);*/
								
								coverageDetails[0] = eb1;
								coverageDetails[1] = eb2;
								coverageDetails[2] = eb3;
								coverageDetails[3] = eb4;
								if (amount != null) {
									coverageDetails[4] = amount.toString();
								} else {
									coverageDetails[4] = null;
								}
								return coverageDetails;
							}
						}
							}
					}
				}
			}

		} catch (EDIParseException ex) {
			ex.printStackTrace();
		}

		return coverageDetails;
	}

	public static DataObject getCoverageDetailsObject(byte[] data) {
		String[] details = getCoverageTypeDetails(data);
		DataObject coverageTypeDetails = IndustrySupportUtil
				.createDO(
						"http://www.ibm.com/xmlns/prod/websphere/fabric/2009/12/healthcare/payor/schema/BenefitEligibilityDetails",
						"CoverageTypeDetails");
		if(details == null) {
			coverageTypeDetails.setString("status", "on");
			coverageTypeDetails.setString("message", "Patient Id is Invalid");
			coverageTypeDetails.setString("benefitCoverageLevel", "-");
			coverageTypeDetails.setString("eligibleForBenefits", "-");
			coverageTypeDetails.setString("serviceType", "-");
			coverageTypeDetails.setString("deductibleAmount", "-");
			coverageTypeDetails.setString("copayAmount", "-");
			coverageTypeDetails.setString("insuranceType", "-");
			return coverageTypeDetails;
		}
		else {
		coverageTypeDetails.setString("eligibleForBenefits",
				getEligibleForBenefits(details[0]));
		if (details[1] == "" || details[1] == null)
			coverageTypeDetails.setString("benefitCoverageLevel", "-");
		else
			coverageTypeDetails.setString("benefitCoverageLevel",getEligibleForBenefits(details[1]));
			
		if (details[2] == "" || details[2] == null)
			coverageTypeDetails.setString("serviceType", "-");
		else
			coverageTypeDetails.setString("serviceType", getEligibleForBenefits(details[2]));
		
		if (details[3] == "" || details[3] == null)
			coverageTypeDetails.setString("insuranceType", "-");
		else
			coverageTypeDetails.setString("insuranceType", getEligibleForBenefits(details[3]));
		
		if (details[0].equals("A") || details[0].equals("B")) {
			coverageTypeDetails.setString("copayAmount", "$" + details[4]);
			coverageTypeDetails.setString("deductibleAmount", "-");
		} else if (details[0].equals("C")) {
			coverageTypeDetails.setString("copayAmount", "-");
			coverageTypeDetails.setString("deductibleAmount", "$" + details[4]);
		}
		if(details[0].equals("6")){
			coverageTypeDetails.setString("copayAmount", "-");
			coverageTypeDetails.setString("deductibleAmount", "-");
		}
		coverageTypeDetails.setString("message", "Valid");
		return coverageTypeDetails;
		}
	}

	private static String getEligibleForBenefits(String key) {
		InputStream is = BPELUtility.class
				.getResourceAsStream("beinformation.properties");
		Properties properties = new Properties();
		try {
			properties.load(is);
			return properties.getProperty(key);
		} catch (IOException e) {
			LOG.error(e.getMessage());
		}
		return null;
	}

	private static Loop2000C[] getSubscriberLoop(Hipaa270Document hipaa270Doc) {
		Loop2000A[] loop2000A = hipaa270Doc.getChildren();
		if (!ArrayUtils.isEmpty(loop2000A)) {
			for (int i = 0; i < loop2000A.length; i++) {
				Loop2000B[] loop2000B = loop2000A[i].getChildren();
				if (!ArrayUtils.isEmpty(loop2000B)) {
					for (int j = 0; j < loop2000B.length; j++) {
						Loop2000C[] loop2000c = loop2000B[j].getChildren();
						return loop2000c;
					}
				}
			}
		}
		LOG.error("Subscriber loop not found.");
		return null;
	}
	
	public static DataObject getContext(){
		commonj.sdo.DataObject context = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextTypeExt");
/*		System.out.println("Created context DataObject" +context);*/

		commonj.sdo.DataObject contextProperty_01 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_02 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_03 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_04 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_05 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_06 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		commonj.sdo.DataObject contextProperty_07 = com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil.createDO("http://docs.oasis-open.org/wscaf/2005/10/wsctx","contextIdentifierType");
		
		contextProperty_01.setString("name","http://www.webifysolutions.com/2005/10/catalog/governance#Environment");
		contextProperty_01.setString("value","fc://fabric/governance/default-inst#default");
		contextProperty_02.setString("name","http://www.webifysolutions.com/context/debug");
		contextProperty_02.setString("value","true");
		contextProperty_03.setString("name","http://www.webifysolutions.com/context/interface");
		contextProperty_03.setString("value","http://BESubmission/BenefitsAndEligibilitySubmissionProcessInterface#BenefitsAndEligibilitySubmissionProcess");
		contextProperty_04.setString("name","http://www.webifysolutions.com/2005/10/catalog/governance#FabricProject");
		contextProperty_04.setString("value","fc://fabric/governance/fabricproject-inst#0dceb16c-bafa-473f-b289-39897c8225a6");
		
		java.util.List propertyList = new java.util.ArrayList() ;
		propertyList.add(contextProperty_01);
		propertyList.add(contextProperty_02);
		propertyList.add(contextProperty_03);
		propertyList.add(contextProperty_04);
	
		context.setString("timestamp","122");
		context.setList("property",propertyList);
		
		return context;
	}
}